/* global $ */
/* global YT */
$(function() {
  var VIEW = {
    local: getVar('#hp-view-local'),
    online: getVar('#hp-view-online')
  };
  var BUTTON = {
    next: {
      altText: getVar('#hp-next-alt'),
      tooltip: getVar('#hp-next-tooltip')
    },
    prev: {
      altText: getVar('#hp-previous-alt'),
      tooltip: getVar('#hp-previous-tooltip')
    },
    forward: {
      altText: getVar('#hp-forward-alt'),
      tooltip: getVar('#hp-forward-tooltip')
    },
    back: {
      altText: getVar('#hp-back-alt'),
      tooltip: getVar('#hp-back-tooltip')
    },
    highlights: {
      tooltip: getVar('#hp-highlights-tooltip')
    },
    print: {
      tooltip: getVar('#hp-print-tooltip')
    }
  };
  var SEARCH = {
    first: getVar('#hp-search-all'),
    placeholder: $('.search-field:last').attr('placeholder'),
    delay: 1400
  };
  var RESPONSIVE_WIDTH = 640;
  var IPN_SCROLL_Y = 200;
  var VERSIONLABEL = getVar('#hp-version-label');
  var VERSION = getVar('#hp-help-version');
  var WATCHMOVIE = getVar('#hp-watch-movie-visible').toLowerCase()==='yes';
  var WATCHMOVIETOOLTIP = getVar('#hp-watch-movie-tooltip');
  var DEBUG_ENABLED = true;
  var log = function() {};
  var warn = function() {};
  var error = function() {};
  var isIE = /*@cc_on!@*/false || !!document.documentMode;
  var UrlContentIndex = document.location.href.indexOf('Content');
  var UrlContentBase = document.location.href.substring(0, UrlContentIndex);
  var tocTree = [];
  var YTplayer = null;
  
  log('TopNav::Initializing');
  if (DEBUG_ENABLED)
    init();
  else
    try {
      init();
    } catch(e) {
      error(e.message);
    }
  
  function init() {
    if (!DEBUG_ENABLED)
      window.onerror = function(msg, url, line) {
        error({msg: msg, url: url, line: line});
        return true;
      }

    log('TopNav::Checking debug mode');
    setDebugMode();
    log('TopNav::Updating elements');
    updateElements();
    log('TopNav::Updating search filter');
    updateSearchFilter();
    log('TopNav::Updating toolbar');
    updateToolbar();
    log('TopNav::Updating highlights');
    updateHighlights();
    /*log('TopNav::Updating sidebar menu');
    updateSidebarMenu();*/
    log('TopNav::Generating version menu');
    generateVersionMenu();
    log('TopNav::Generating settings menu');
    generateSettingsMenu();
    log('TopNav::Configuring responsive design');
    responsiveDesign();
    log('TopNav::Showing elements');
    showElements();
    log('TopNav::Resizing image maps');
    callAttempt({
      callback: function() {
        $('map').imageMapResize();
      },
      interval:400,
      maxAttempts: 5
    });
    log('TopNav::Completed');
  }

  function setDebugMode() {
    if (typeof console!=='undefined') {
      log = (DEBUG_ENABLED) ? console.log.bind(console) : function() {};
      warn = (DEBUG_ENABLED) ? console.warn.bind(console) : function() {};
      error = (DEBUG_ENABLED) ? console.error.bind(console) : function() {};
    }
  }

  function updateElements() {
    var skinName = '<span class="skin-name">'+getVar('#hp-help-name')+'</span>';
    var skinShortName = '<span class="skin-short-name">'+getVar('#hp-help-short-name')+'</span>';

    $('.logo')
      .html(skinName+skinShortName)
      .wrap('<div class="logo-wrapper"><div class="logo-inner"><p></p></div></div>');
    $('.logo-wrapper')
      .prependTo('.tab-bar');
    document.querySelector('.inner-wrap')
      .appendChild(document.querySelector('.footer'));
    $('.tab-bar .navigation-wrapper .navigation')
      .wrap('<div></div>');
    $('#TopNav.home .main-section')
        .css('padding-bottom',
          (parseInt($('#TopNav .footer').height()) + 14) + 'px');
    var cats = $('ul.navigation > li').length;
    $('ul.navigation > li:first-child')
      .css('margin-left', (8.25 * (6 - cats)) + '%');
    $('ul.navigation > li > a')
      .each(function() {
        this.href='#';
      });
    updateCatoriesTooltips();
    $('.other .nav-search.row.outer-row')
      .wrap('<div class="search-wrapper"></div>');
    $('.other .search-wrapper')
      .appendTo('.logo-wrapper');
    $('.other[hpidocintopic] .search-bar.search-bar-container.needs-pie > *')
      .wrapAll('<div></div>');
      
    $('img.play').wrap('<div class="play"></div>');

    $('#hp-feedbackDimmedDiv')
      .appendTo('body');
    $('#hp-feedbackDiv')
      .appendTo('body');
      
    /* Home page movie */
    configureWatchMovie();

    fixTocTree('#TopNav ul.off-canvas-list li.has-submenu a');
    generateInPageNav();
  }
  
  function updateCatoriesTooltips() {
    setTimeout(function() {
      $('ul.navigation > li > a').each(function() {
        var $el = $(this);
        var title = $el.text();
        var $c = $el
                  .clone()
                  .css({
                    display: 'inline',
                    width: 'auto',
                    visibility: 'hidden'
                  })
                  .appendTo('body');
        
        if($c.width() > $el.width()) {
          $el.attr('data-title', title);
        } else {
          $el.removeAttr('data-title');
        }
        $c.remove();
      });
    }, 1000);
  }
  
  function configureWatchMovie() {
    if (WATCHMOVIE) {
      log('WATCHMOVIETOOLTIP::',WATCHMOVIETOOLTIP);
      /*$('#TopNav.home .watch-movie,#TopNav.home .watch-movie-responsive')*/
      $('#TopNav.home .search-area-right img')
        .attr('title', WATCHMOVIETOOLTIP) 
        .attr('alt', WATCHMOVIETOOLTIP); 
          
      var movieSrc = $('#TopNav.home .movie-wrapper iframe').attr('src');
      
      movieSrc += ((movieSrc.indexOf('?')===-1) ? '?' : '&');
      movieSrc += 'enablejsapi=1';
      movieSrc += (document.location.protocol==='file:') ? '' : '&origin='+document.location.origin;
      
      $('#TopNav.home .movie-wrapper iframe')
        .attr('id', 'watch-movie')
        .attr('src', movieSrc)
        .removeAttr('height')
        .removeAttr('width')
        .css({
          height: '100%',
          width: '100%'
        });
        
      window.onYouTubePlayerAPIReady = function() {
        log('EVENT::onYouTubePlayerAPIReady');
        YTplayer = new YT.Player(
          'watch-movie',
          {
            events: {
              onReady: function() {
                log('EVENT::YTplayer.onReady');
                $('#TopNav.home .movie-wrapper button')
                  .click(function() {
                    log('EVENT::Pause movie');
                    YTplayer.pauseVideo();
                  });
                $('#TopNav.home .search-area-right img')
                  .click(function(e) {
                    e.preventDefault();
                    log('EVENT::Play movie');
                    YTplayer.playVideo();
                  });
              }
            }
          }
        );
        $('#TopNav.home .movie-wrapper button')
          .click(function() {
            log('EVENT::Hide movie');
            $('#TopNav.home .movie-overlay')
              .addClass('hidden');
          });
        $('#TopNav.home .search-area-right img')
          .click(function(e) {
            e.preventDefault();
            log('EVENT::Show movie');
            $('#TopNav.home .movie-overlay')
              .removeClass('hidden');
          });
        $('#TopNav.home .search-area-right').removeClass('hidden');   
      }
    }
  }
  
  function fixTocTree(sel) {
    setTimeout(function() {
      $(sel)
        .on("mousedown", function() {
          log('Menu item clicked');
          var menu = '.left-submenu.move-right:last';
          fixTocTree(menu);
          $(menu).scrollTop(0);
        });
    }, 1000);
  }
  
  function generateInPageNav() {
    var id = setInterval(function() {
      if (!$('ul.hpMiniTOCItems li').length)
        return;
      else
        clearInterval(id);
      
      if ($('ul.hpMiniTOCItems')) {
        $('ul.hpMiniTOCItems')
          .clone()
          .removeClass('hpMiniTOCItems')
          .addClass('ipn-menu')
          .appendTo('body')
          .wrap('<div class="ipn open"></div>');
        $('ul.ipn-menu li')
          .each(function() {
            var $this = $(this);
            var text = $this.text();
            var href = $('a', this).attr('href');
            $this.attr('href', href);
            $this.html('<span>'+text+'</span><i></i>');
          })
          .click(function() {
            document.location.hash = $(this).attr('href');
            $(this).addClass('active').siblings().removeClass('active');
            if ($("html.web").length) return;
            setTimeout(function() {
              window.scrollTo(0,$(window).scrollTop()-71);
            }, 250);
          });
        $('<li href="#hp-top-anchor" class="ipn-top"><span>Top</span><i></i></li>')
          .click(function() {
            $(window).scrollTop(0);
          })
          .appendTo('ul.ipn-menu');
        $('<li class="ipn-toggle"><span>Close</span><i></i></li>')
          .click(function() {
            setTimeout(function() {
              $('#TopNav .ipn')
                .removeClass('open')
                .on('click', function() {
                  $(this).off('click');
                  $(this).addClass('open');
                });
            }, 0);
          })
          .prependTo('ul.ipn-menu');
        if ($(window).width() > RESPONSIVE_WIDTH &&
            $(window).scrollTop() >= IPN_SCROLL_Y)
          $('#TopNav .ipn').addClass('show');
        $("h2:in-viewport:first").each(function() {
          $('ul.ipn-menu li[href=#'+this.id+']')
            .addClass('active').siblings().removeClass('active');
        });
        $(window).scroll(function() {
          if ($(window).width() > RESPONSIVE_WIDTH &&
              $(window).scrollTop() >= IPN_SCROLL_Y)
            $('#TopNav .ipn').addClass('show');
          else
            $('#TopNav .ipn').removeClass('show');
            
          $("h2:in-viewport:first").each(function() {
            $('ul.ipn-menu li[href=#'+this.id+']')
              .addClass('active').siblings().removeClass('active');
          });
        });
      }
    }, 500);
  }

  function updateSearchFilter() {
    /* optional input attributes results=5 autocomplete=on */
    var filterText = $('.search-filter:last').html();
    $('.search-filter:last').html('<div class="search-filter-label"></div>'+filterText);
    setTimeout(function() {
      $('#TopNav.home .search-area-bar').show();
      $('.other .nav-search.row.outer-row').show();
    }, 100);

    var searchFilter = getValue('searchFilter');
    if (!searchFilter) searchFilter = SEARCH.first;

    setTimeout(function() {
      $('.search-filter:last > .search-filter-content > ul > li:first-child')
        .text(SEARCH.first);
    }, SEARCH.delay);
  }

  function setSearchLabel(label) {
    if ($('.search-filter-label:last').text() !== label) {
      $('.search-filter-label:last').text(label);
      setValue('searchFilter', label);
    }

    var wFilter = $('.search-filter-label:last').width();
    if (!wFilter) {
      log('setSearchLabel::recursion:width is 0');
      setTimeout(function() {setSearchLabel(label)}, 100);
      return;
    }

    var offset = 20;
    var wAvail =
      $('.search-bar:last').width() -
      $('.search-submit:last').width() - wFilter;

    $('.search-field:last').width(wAvail-offset);
    $('.search-field:last').css('margin-left', wFilter+offset+'px');
    $('.search-field:last').attr('placeholder', SEARCH.placeholder);

    if ($('#TopNav.home .search-area-bar').css('opacity')==='0' ||
        $('#TopNav.other .search-wrapper').css('opacity')==='0') {
      $('#TopNav.other .search-wrapper')
        .css('opacity', '1');
      $('#TopNav.home .search-area-bar')
      .css('opacity', 1);
      $('.search-filter:last > .search-filter-content > ul > li').click(function() {
        setSearchLabel($(this).text());
        log('search-filter option clicked: ', $(this).text());
      });
    }
  }

  function updateToolbar() {
    $('#proxy-topic-toolbar')
      .appendTo('.tab-bar-section .navigation-wrapper div');

    $('.other .buttons.clearfix.topicToolbarProxy .remove-highlight-button')
      .attr('title', BUTTON.highlights.tooltip);
    $('.other .buttons.clearfix.topicToolbarProxy .print-button')
      .attr('title', BUTTON.print.tooltip);
    $('.other .buttons.clearfix.topicToolbarProxy .button')
      .each(function() {
        var $this = $(this);
        $('<span class="topic-toolbar-tooltip">'+this.title+'</span>')
          .appendTo($this);
        if (!isIE) this.title = '';
        $this.addClass('tooltips');
      });
    $('.other .buttons.clearfix.topicToolbarProxy .expand-all-button')
      .click(function() {
        this.lastChild.innerHTML = this.title;
      });

    var buttons = [
      {
        class: 'next-topic-button',
        alt: BUTTON.next.altText,
        text: BUTTON.next.tooltip,
        click: function() {
          var next = $('.other #hp-sidebar-menu .sidebar li > a.selected');
          if (next.next().prop("tagName")=="UL")
            next = next.next().children('li:first').children('a:first');
          else
            next = next.parent().next().children('a:first');

          if (next.length)
            location.href = next.attr('href');
        }
      },
      {
        class: 'previous-topic-button',
        alt: BUTTON.prev.altText,
        text: BUTTON.prev.tooltip,
        click: function() {
          var prev = $('.other #hp-sidebar-menu .sidebar li > a.selected');
          if (prev.parent().prev().prop("tagName")=="LI")
            prev = prev.parent().prev().children('a:first');
          else
            prev = prev.parent().parent().prev();

          if (prev.length)
            location.href = prev.attr('href');
        }
      },
      {
        class: 'forward-button',
        alt: BUTTON.forward.altText,
        text: BUTTON.forward.tooltip,
        click: function() {window.history.forward();}
      },
      {
        class: 'back-button',
        alt: BUTTON.back.altText,
        text: BUTTON.back.tooltip,
        click: function() {window.history.back();}
      }
    ];

    for (var i=0; i<buttons.length; i++) {
      var btn = $('.other .buttons.clearfix.topicToolbarProxy .print-button').clone();
      btn.removeClass('print-button').addClass(buttons[i].class);
      btn.children('img').attr('alt', buttons[i].alt);
      btn.children('span').text(buttons[i].text);
      if (isIE) btn.attr('title', buttons[i].text);
      btn.click(buttons[i].click);
      btn.prependTo('.other .buttons.clearfix.topicToolbarProxy .button-group-container-left');
    }
    updateToolbarToolTips();
    $(window).resize(function() {
      updateToolbarToolTips();
      if ($(window).width() > RESPONSIVE_WIDTH &&
          $(window).scrollTop() >= IPN_SCROLL_Y)
        $('#TopNav .ipn').addClass('show');
      else
        $('#TopNav .ipn').removeClass('show');
    });
  }
  
  function updateToolbarToolTips()
  {
    setTimeout(function() {
      $('.other .buttons.clearfix.topicToolbarProxy .button.tooltips span')
        .each(function() {
          this.style.right = '-' + (this.offsetWidth / 2 - 12) + 'px'
      });
    }, 500);
  }

  function showElements() {
    if (WATCHMOVIE)
      $('#TopNav.home .search-area-right').removeClass('hidden');
    $('.other .sidebar').removeClass('hidden');
    $('#proxy-breadcrumbs').removeClass('hidden');
    $('#proxy-body').removeClass('hidden');
    $('#proxy-topic-toolbar').removeClass('hidden');
  }

  function updateHighlights() {
    $('.home .highlight-area table td')
      .each(function(i) {
        var $imgs = $('img', this);

        if ($imgs.length) {
          var $img0 = $($imgs[0]);
          var $img1 = $($imgs[1]);

          $img0.attr('onmouseover', 'this.src="'+$img1.attr('src')+'"');
          $img0.attr('onmouseout', 'this.src="'+$img0.attr('src')+'"');
          $img1.next().attr('onmouseover', 'this.previousSibling.src="'+$img1.attr('src')+'"');
          $img1.next().attr('onmouseout', 'this.previousSibling.src="'+$img0.attr('src')+'"');
          $img1.remove();
        }
      });
  }
  
  function updateSidebarMenu() {
    getTocTree(0, 0, $('#hp-toc-tree > ul > li'));
    
    var tLevel = -1;
    $('#hp-sidebar-menu .has-children li > a')
      .each(function(index, value) {
        var found = false;
        var pText = $(value).parents('.has-children').find('> a').text();
        if (tLevel===-1)
          for (var i=tocTree.length-1; i>=0 && !found; i--) {
            for (var j=0; j<tocTree[i].length && !found; j++) {
              if (tocTree[i][j].text===$(value).text()) {
                var pLevel = tocTree[i][j].pLevel;
                var pIndex = tocTree[i][j].pIndex;
                if (tocTree[pLevel][pIndex].text===pText) {
                  found = true;
                  tLevel = i;
                  if (tocTree[i][j].hasChildren &&
                     !$(value).parent().hasClass('has-children')) {
                    $(value).parent().addClass('has-children');
                  }
                  if ($(value).parent('.has-children').find('.selected').length)
                    $(value).addClass('child-selected');
                }
              }
            }
          }
        else
          for (var j=0; j<tocTree[tLevel].length && !found; j++) {
            if (tocTree[tLevel][j].text===$(value).text()) {
              var pLevel = tocTree[tLevel][j].pLevel;
              var pIndex = tocTree[tLevel][j].pIndex;
              if (tocTree[pLevel][pIndex].text===pText) {
                found = true;
                if (tocTree[tLevel][j].hasChildren &&
                   !$(value).parent().hasClass('has-children')) {
                  $(value).parent().addClass('has-children');
                }
                if ($(value).parent('.has-children').find('.selected').length)
                  $(value).addClass('child-selected');
              }
            }
          }
    });
    if ((tLevel -= 1) < 0) return;
    $('#hp-sidebar-menu li > a')
      .each(function(index, value) {
        var found = false;
        for (var j=0; j<tocTree[tLevel].length && !found; j++) {
          if (tocTree[tLevel][j].text===$(value).text()) {
            found = true;
            if (tocTree[tLevel][j].hasChildren &&
                !$(value).parent().hasClass('has-children')) {
              $(value).parent().addClass('has-children');
            }
            if ($(value).parent('.has-children').find('.selected').length) {
              $(value).addClass('child-selected');
            }
          }
        }
      });
  }
  
  function getTocTree(pLevel, pIndex, list) {
    if (tocTree.length === pLevel) tocTree.push([]);
    list.each(function(index, value) {
      tocTree[pLevel].push({
        pLevel: pLevel-1,
        pIndex: pIndex,
        hasChildren: $(value).hasClass('has-children'),
        text: $('> a',value).text()
      });
      if ($(value).hasClass('has-children'))
        getTocTree(pLevel+1, tocTree[pLevel].length-1, $('> ul > li', value));
    });
  }

  function generateVersionMenu() {
    if (!VERSION) return;

    //var verMenu = $('<ul class="verLangMenu responsive"><li><span>' + VERSIONLABEL + ' ' + VERSION + '</span></li></ul>').appendTo('.logo-wrapper .logo');
    var verLangMenu = $('<ul class="verLangMenu normal"><li><span>' + VERSIONLABEL + ' ' + VERSION + '</span></li></ul>').appendTo('.logo-wrapper .logo');

    if (location.protocol !== 'file:') {
      function processVersionFile(details) {
        var verFilePath =
          details.baseURL +
          '../'.repeat(details.tryLevel) +
          details.verFileName;

        log('processVersionFile::verFilePath:',verFilePath);
        $.getScript(verFilePath)
          .done(function() {
            var vlmData = {};
            var curr = {
              ver: parts.slice(-details.tryLevel)[0],
              lang: parts.slice(-(details.tryLevel+1))[0]
            };
                
            log('processVersionFile::curr:',curr);
            for (var i=0; i<vlmPaths.length; i++) {
                parts = vlmPaths[i].split('/');

                if (!vlmData[parts[1]])
                    vlmData[parts[1]] = [];
                vlmData[parts[1]].push({'lang': parts[0], 'path': vlmPaths[i]});
            }

            if (!$.isEmptyObject(vlmData)) {
                //verMenu += '<ul>';
                $('ul.verLangMenu.normal').addClass('online');
                verLangMenu = '<ul>';
                for (var x in vlmData) {
                  verLangMenu += '<li>';
                  //verMenu += '<li>';
                  if (vlmData[x].length === 1) {
                    var item = vlmData[x].pop();
                    verLangMenu += '<a href="'+((x===curr.ver||x===vlmLatest)?'#" class="selected"':vlmBaseURL+item.path+'"')+'>'+x+'</a>';
                    //if (item.lang==='en') verMenu += '<a href="'+((x===curr.ver)?'#" class="selected"':vlmBaseURL+item.path+'"')+'>'+x+'</a>';
                  } else {
                    verLangMenu += '<a href="#"'+((x===curr.ver||x===vlmLatest)?' class="selected"':'')+'>'+x+'</a>';
                    verLangMenu += '<ul>';
                    while (vlmData[x].length) {
                      var item = vlmData[x].pop();
                      verLangMenu += '<li><a href="'+
                        (((x===curr.ver||x===vlmLatest) && item.lang===curr.lang)?'#" class="selected"':vlmBaseURL+item.path+'"')+
                        '>'+vlmLangs[item.lang]+'</a></li>';
                      //if (item.lang==='en') verMenu += '<a href="'+((x===curr.ver)?'#" class="selected"':vlmBaseURL+item.path+'"')+'>'+x+'</a>';
                    }
                    verLangMenu += '</ul>';
                  }
                  verLangMenu += '</li>';
                  //verMenu += '</li>';
                }
                verLangMenu += '</ul>';
                //verMenu += '</ul></li></ul>';
                //verMenu += '</li></ul>';
                
                //$('.logo-wrapper .logo').append(verMenu);
                $('ul.verLangMenu.normal li').append(verLangMenu);
            }
          })
          .fail(function() {
            log('processVersionFile::Next tryLevel:',details.tryLevel+1);
            if (++details.tryLevel <= details.maxLevel)
              processVersionFile(details);
          });
      }

      String.prototype.repeat = function(num) {
        return new Array(isNaN(num)? 1 : ++num).join(this);
      }
      var parts = UrlContentBase.split('/');

      processVersionFile({
        tryLevel: 2,
        maxLevel: parts.length-4,
        baseURL: UrlContentBase,
        verFileName: '_TopNav_version-language-data.js'
      });
    }
  }

  function generateSettingsMenu() {
    var switchStatus = getValue('switchStatus');
    if (switchStatus && switchStatus !== 'other') {
      addSwitch({
        switchStatus: (switchStatus === 'offline') ? VIEW.online : VIEW.local,
        switchPath: getValue('switchPath'),
        currentPath: getValue('currentPath')
      });
    }
  }
  
  function responsiveDesign() {
    $(window).resize(function() {
      if ($('body').innerWidth() > 640)
        $('.off-canvas-wrap.move-right')
          .removeClass('move-right');
      
      $('#TopNav .main-section')
        .css('padding-bottom', 
          (parseInt($('#TopNav .footer').height()) + 14) + 'px');

      updateCatoriesTooltips();          
    });
    $('#TopNav nav.tab-bar .row.outer-row .menu-icon')
      .click(function() {
        if ($('#TopNav nav.tab-bar .row.outer-row .menu-icon').attr('aria-expanded')!=='true') {
          window.scrollTo(0,0);
        }
      });
    $('#TopNav.other .search-wrapper')
      .addClass('Responsive_Closed');
    $('#search-responsive-back')
      .click(function() {
        $('#TopNav.other .search-wrapper')
          .toggleClass('Responsive_Closed Responsive_Open');
        setTimeout(function() {
          $('#search-responsive-wrapper')
            .removeClass('Responsive_Hide');
          $('#TopNav nav.tab-bar .row.outer-row .menu-icon')
            .removeClass('Responsive_Hide');
          $('#TopNav .inner-wrap .logo-wrapper a.logo')
            .removeClass('Responsive_Hide');
        }, 450);
      })
      .wrap('<div id="search-wrapper-back"></div>');
    $('#TopNav.other #search-wrapper-back')
      .appendTo('#TopNav.other .search-wrapper .search-bar');
    $('#search-responsive')
      .appendTo('.logo-wrapper')
      .wrap('<div id="search-responsive-wrapper"></div>');
    $('#search-responsive-wrapper')
      .click(function() {
        $('#search-responsive-wrapper')
          .addClass('Responsive_Hide');
        $('#TopNav nav.tab-bar .row.outer-row .menu-icon')
          .addClass('Responsive_Hide');
        $('#TopNav .inner-wrap .logo-wrapper a.logo')
          .addClass('Responsive_Hide');
        $('#TopNav.other .search-wrapper')
          .toggleClass('Responsive_Closed Responsive_Open');
      })
  }

  function addSwitch(options) {
    log('TopNav::options:', options);
    $('<div class="settings-wrapper">' +
      '<div id="hp-settings-menu">' +
      '<ul>' +
      '<li class="switchStatus">'+options.switchStatus+'</li>' +
      '</ul></div></div>')
      .appendTo('.logo-wrapper');
    $('#hp-settings-menu .switchStatus')
      .click(function() {
        log('online state: ' + (($(this).text()==='View offline') ? 'online' : 'offline'));
        var url = options.switchPath +
          '#topic=' + document.location.href.substring(UrlContentIndex) +
          '?path=' + options.currentPath;
        log('url::', url);
        document.location.replace(url);
      });
  }

  function getValue(name) {
    if (typeof(Storage) !== "undefined" && typeof(sessionStorage) !== "undefined")
      return sessionStorage[name] || null;
    else if (typeof(document.cookie) !== "undefined")
      return getCookie(name) || null;
    else
      return null;
  }

  function setValue(name, value) {
    if (typeof(Storage) !== "undefined" && typeof(sessionStorage) !== "undefined")
      sessionStorage[name] = value;
    else if (typeof(document.cookie) !== "undefined")
      setCookie(name, value, 1);
  }

  function getCookie(cname) {
    var name = cname + "=";
    var ca = document.cookie.split(';');
    for (var i = 0; i < ca.length; i++) {
      var c = ca[i];
      while (c.charAt(0) == ' ') c = c.substring(1);
      if (c.indexOf(name) == 0) return c.substring(name.length, c.length);
    }
    return "";
  }

  function setCookie(cname, cvalue, exdays) {
    var d = new Date();
    d.setTime(d.getTime() + (exdays*24*60*60*1000));
    var expires = "expires="+d.toUTCString();
    document.cookie = cname + "=" + cvalue + "; " + expires;
  }
  
  function callAttempt(params) {
    (function(c,s,i,m) {
      var a = 0;
      var m = m || 10;
      console.log('callAttempt::params: ', {c:c,s:s,i:i,m:m});
      var id = setInterval(function() {
        console.log('callAttempt::attempt: '+(a+1)+' of '+m);
        if (++a<m) {
          if (s && !$(s).length) {
            console.log('callAttempt::return');
            return;
          } else if (s && $(s).length) {
            console.log('callAttempt::clear+success+callback');
            clearInterval(id);
            c();
            return;
          } else {
            console.log('callAttempt::callback');
            c();
          }
        } else {
          console.log('callAttempt::clear');
          clearInterval(id);
          if (s && $(s).length) {
            console.log('callAttempt::finalAttempt+success+callback');
            c();
          }
        }
      }, i);
    })(params.callback,params.condition,params.interval,params.maxAttempts);
  }
  
  function getVar(vname) {
    return $(vname).text();
  }
});

(function($){$.belowthefold=function(element,settings){var fold=$(window).height()+$(window).scrollTop();return fold<=$(element).offset().top-settings.threshold;};$.abovethetop=function(element,settings){var top=$(window).scrollTop();return top>=$(element).offset().top+$(element).height()-settings.threshold;};$.rightofscreen=function(element,settings){var fold=$(window).width()+$(window).scrollLeft();return fold<=$(element).offset().left-settings.threshold;};$.leftofscreen=function(element,settings){var left=$(window).scrollLeft();return left>=$(element).offset().left+$(element).width()-settings.threshold;};$.inviewport=function(element,settings){return!$.rightofscreen(element,settings)&&!$.leftofscreen(element,settings)&&!$.belowthefold(element,settings)&&!$.abovethetop(element,settings);};$.extend($.expr[':'],{"below-the-fold":function(a,i,m){return $.belowthefold(a,{threshold:0});},"above-the-top":function(a,i,m){return $.abovethetop(a,{threshold:0});},"left-of-screen":function(a,i,m){return $.leftofscreen(a,{threshold:0});},"right-of-screen":function(a,i,m){return $.rightofscreen(a,{threshold:0});},"in-viewport":function(a,i,m){return $.inviewport(a,{threshold:0});}});})(jQuery);